@@ -320,25 +320,19 @@ Thread::returnvalue Thread::ForceTerminate()
 
 bool Thread::IsRunning()
 {
-    CriticalSectionLocker CSL(C);
-    const bool ToReturn=State==State_Running || State==State_Terminating;
-    return ToReturn;
+    return State==State_Running;
 }
 
 //---------------------------------------------------------------------------
 bool Thread::IsTerminating()
 {
-    CriticalSectionLocker CSL(C);
-    const bool ToReturn=State==State_Terminating;
-    return ToReturn;
+    return State==State_Terminating;
 }
 
 //---------------------------------------------------------------------------
 bool Thread::IsExited()
 {
-    CriticalSectionLocker CSL(C);
-    const bool ToReturn=State==State_New || State==State_Terminated;
-    return ToReturn;
+    return State==State_Terminated;
 }
 
 //***************************************************************************
@@ -524,6 +518,9 @@ Thread::returnvalue Thread::RequestTerminate()
 Thread::returnvalue Thread::ForceTerminate()
 {
     //Terminating (not clean)
+    #if !defined(__ANDROID_API__)
+    pthread_cancel((pthread_t)ThreadPointer);
+    #endif
 
     //Configuring
     State=State_Terminated;
@@ -538,33 +535,28 @@ Thread::returnvalue Thread::ForceTerminate()
 //---------------------------------------------------------------------------
 bool Thread::IsRunning()
 {
-    CriticalSectionLocker CSL(C);
-    const bool ToReturn=State==State_Running;
-    return ToReturn;
+    return State==State_Running;
 }
 
 //---------------------------------------------------------------------------
 bool Thread::IsTerminating()
 {
-    CriticalSectionLocker CSL(C);
-    const bool ToReturn=State==State_Terminating;
-    return ToReturn;
+    return State==State_Terminating;
 }
 
 //---------------------------------------------------------------------------
 bool Thread::IsExited()
 {
-    CriticalSectionLocker CSL(C);
-    const bool ToReturn=State==State_New || State==State_Terminating;
-    return ToReturn;
+    return State==State_Terminated;
 }
 
 //***************************************************************************
 // Communicating
 //***************************************************************************
 
-void Thread::Sleep(size_t)
+void Thread::Sleep(size_t Millisecond)
 {
+    usleep(Millisecond*1000);
 }
 
 void Thread::Yield()
